# Generated by Django 3.2.4 on 2021-07-15 07:48

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pretixbase", "0195_auto_20210622_1457"),
        ("pretix_vacc_autosched", "0003_itemconfig_max_days"),
    ]

    operations = [
        migrations.AddField(
            model_name="itemconfig",
            name="second_item",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="pretixbase.item",
            ),
        ),
    ]



from django.urls import path, re_path

from . import api
from .views import SelfServiceBookingView, SelfServiceIndexView, SettingsView

urlpatterns = [
    path(
        "control/event/<str:organizer>/<str:event>/settings/vacc_autosched/",
        SettingsView.as_view(),
        name="settings",
    ),
    path(
        "api/v1/organizers/<str:organizer>/events/<str:event>/items/<str:item>/vacc_autosched/",
        api.ItemView.as_view(),
        name="api.item",
    ),
]

event_patterns = [
    path(
        "2nd/<str:order>/",
        SelfServiceBookingView.as_view(),
        name="second.booking",
    ),
    re_path(
        r"^2nd/?$",
        SelfServiceIndexView.as_view(),
        name="second.index",
    ),
]



# Generated by Django 3.0.12 on 2021-03-10 12:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pretixbase", "0175_orderrefund_comment"),
        ("pretix_vacc_autosched", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="itemconfig",
            name="item",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="vacc_autosched_config",
                to="pretixbase.Item",
            ),
        ),
        migrations.CreateModel(
            name="LinkedOrderPosition",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                (
                    "base_position",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="vacc_autosched_linked",
                        to="pretixbase.OrderPosition",
                    ),
                ),
                (
                    "child_position",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="vacc_autosched_link",
                        to="pretixbase.OrderPosition",
                    ),
                ),
            ],
        ),
    ]



import datetime as dt
import logging
from django.contrib import messages
from django.http import Http404
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.formats import date_format
from django.utils.functional import cached_property
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _
from django.views.generic import FormView
from pretix.base.models import Event, Quota
from pretix.base.services.quotas import QuotaAvailability
from pretix.control.views.event import EventSettingsFormView, EventSettingsViewMixin
from pretix.multidomain.urlreverse import eventreverse
from pretix.presale.views import EventViewMixin

from pretix_vacc_autosched.forms import (
    AutoschedSettingsForm,
    SecondDoseCodeForm,
    SecondDoseOrderForm,
)
from pretix_vacc_autosched.models import LinkedOrderPosition
from pretix_vacc_autosched.tasks import get_for_other_event

logger = logging.getLogger(__name__)


class SettingsView(EventSettingsViewMixin, EventSettingsFormView):
    model = Event
    form_class = AutoschedSettingsForm
    template_name = "pretix_vacc_autosched/settings.html"
    permission = "can_change_settings"

    def get_success_url(self) -> str:
        return reverse(
            "plugins:pretix_vacc_autosched:settings",
            kwargs={
                "organizer": self.request.event.organizer.slug,
                "event": self.request.event.slug,
            },
        )


class SelfServiceMixin:
    def dispatch(self, request, *args, **kwargs):
        if not request.event.settings.vacc_autosched_self_service:
            raise Http404(_("Feature not enabled"))
        return super().dispatch(request, *args, **kwargs)


class SelfServiceIndexView(SelfServiceMixin, EventViewMixin, FormView):
    form_class = SecondDoseCodeForm
    template_name = "pretix_vacc_autosched/self_service_index.html"

    def get_form_kwargs(self):
        result = super().get_form_kwargs()
        result["event"] = self.request.event
        return result

    def form_valid(self, form):
        order = form.cleaned_data.get("order")
        return redirect(
            eventreverse(
                order.event,
                "plugins:pretix_vacc_autosched:second.booking",
                kwargs={"order": order.code},
            )
        )


class SelfServiceBookingView(SelfServiceMixin, EventViewMixin, FormView):
    form_class = SecondDoseOrderForm
    template_name = "pretix_vacc_autosched/self_service_order.html"

    @cached_property
    def order(self):
        return (
            self.request.event.orders.filter(code=self.kwargs["order"])
            .select_related("event")
            .first()
        )

    def dispatch(self, request, *args, **kwargs):
        if not self.order:
            messages.error(
                request,
                _(
                    "We were unable to find a valid ticket with this code, please try again."
                ),
            )
            return redirect(
                eventreverse(
                    self.request.event,
                    "plugins:pretix_vacc_autosched:second.index",
                )
            )

        if self.order.status != self.order.STATUS_PAID or self.order.require_approval:
            messages.error(
                request,
                _(
                    "Scheduling of a second appointment is not available for this ticket since it has not yet been approved or has been canceled."
                ),
            )
            return redirect(
                eventreverse(
                    self.request.event,
                    "plugins:pretix_vacc_autosched:second.index",
                )
            )

        position = (
            self.order.positions.first()
        )  # We assume that there's only one position per order
        if (
            not position.subevent
            or not position.subevent.date_from.date() <= now().date()
        ):
            messages.error(
                request,
                _(
                    "Please do not try to schedule a second appointment before your first appointment is over."
                ),
            )
            return redirect(
                eventreverse(
                    self.request.event,
                    "plugins:pretix_vacc_autosched:second.index",
                )
            )

        config = getattr(position.item, "vacc_autosched_config", None)
        if not config or not config.days or not position.subevent:
            messages.error(
                request,
                _(
                    "Scheduling of a second appointment is not available for this ticket."
                ),
            )
            return redirect(
                eventreverse(
                    self.request.event,
                    "plugins:pretix_vacc_autosched:second.index",
                )
            )

        self.position = position
        link = LinkedOrderPosition.objects.filter(base_position=position).first()
        if link:
            messages.error(
                request,
                _(
                    "A second appointment has already been scheduled for {datetime}. "
                    "The ticket has been sent to you via email to the address used for your first booking."
                ).format(
                    datetime=date_format(
                        link.child_position.subevent.date_from.astimezone(
                            self.request.event.timezone
                        ),
                        "DATETIME_FORMAT",
                    )
                ),
            )
            return redirect(
                eventreverse(
                    self.request.event,
                    "plugins:pretix_vacc_autosched:second.index",
                )
            )

        first_date = position.subevent.date_from.date()
        min_date = max(first_date + dt.timedelta(days=config.days), now().date())
        max_date = first_date + dt.timedelta(
            days=config.max_days if config.max_days is not None else 1
        )

        self.other_event = config.event or position.order.event

        self.target_item, self.target_variation = get_for_other_event(
            position, self.other_event, config.second_item
        )
        subevents = self.other_event.subevents.filter(
            date_from__date__gte=min_date,
            date_from__date__lte=max_date,
        )
        quotas = Quota.objects.filter(
            subevent__in=subevents, items__id=self.target_item.pk
        )
        qa = QuotaAvailability()
        qa.queue(*quotas)
        qa.compute()
        available_subevents = []
        for subevent in subevents:
            qlist = [q for q in quotas if q.subevent_id == subevent.pk]
            if qlist and all(qa.results[q][0] == Quota.AVAILABILITY_OK for q in qlist):
                available_subevents.append(subevent)

        self.subevents = available_subevents

        if not self.subevents:
            messages.error(
                request,
                _(
                    "Unfortunately, there is currently no available slot for a second appointment."
                ),
            )
            return redirect(
                eventreverse(
                    self.request.event,
                    "plugins:pretix_vacc_autosched:second.index",
                )
            )

        return super().dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        result = super().get_form_kwargs()
        result["position"] = self.position
        result["available_subevents"] = self.subevents
        result["event"] = self.other_event
        return result

    def get_context_data(self, **kwargs):
        result = super().get_context_data(**kwargs)
        result["order"] = self.order
        return result

    def form_valid(self, form):
        from .tasks import book_second_dose

        subevent = form.cleaned_data["subevent"]
        order = book_second_dose(
            op=self.position,
            item=self.target_item,
            variation=self.target_variation,
            subevent=subevent,
            original_event=self.request.event,
        )
        if order:
            messages.success(
                self.request,
                _(
                    "Your appointment has been booked. We've sent you the details via email."
                ),
            )
            return render(
                self.request,
                "pretix_vacc_autosched/thanks.html",
                context=self.get_context_data(),
            )
        else:
            messages.error(
                self.request,
                _(
                    "There was an error when booking your second dose, please try again."
                ),
            )
            return self.form_invalid(form)



from django.db import transaction
from pretix.base.models import Event, Item
from rest_framework import serializers, status, views
from rest_framework.generics import get_object_or_404
from rest_framework.response import Response

from pretix_vacc_autosched.models import ItemConfig


class ItemConfigSerializer(serializers.ModelSerializer):
    event = serializers.SlugRelatedField(
        slug_field="slug", queryset=Event.objects.none()
    )

    class Meta:
        model = ItemConfig
        fields = ["event", "days", "second_item", "max_days"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields[
            "event"
        ].queryset = self.instance.item.event.organizer.events.filter(
            has_subevents=True
        )
        self.fields["second_item"].queryset = Item.objects.filter(
            event__organizer=self.instance.item.event.organizer
        )

    def validate(self, data):
        if data.get("second_item") and data.get("event"):
            if data.get("event") != data.get("second_item").event:
                raise serializers.ValidationError("Item does not exist in event")
        return data


class ItemView(views.APIView):
    permission = "can_change_items"

    def get(self, request, *args, **kwargs):
        item = get_object_or_404(request.event.items.all(), pk=kwargs.get("item"))

        try:
            instance = item.vacc_autosched_config
        except ItemConfig.DoesNotExist:
            return Response({})
        else:
            s = ItemConfigSerializer(instance=instance)
            return Response(s.data)

    def delete(self, request, *args, **kwargs):
        item = get_object_or_404(request.event.items.all(), pk=kwargs.get("item"))

        try:
            instance = item.vacc_autosched_config
        except ItemConfig.DoesNotExist:
            return Response({})
        else:
            with transaction.atomic():
                instance.delete()
                item.log_action(
                    "pretix.event.item.changed", user=self.request.user, data={}
                )
            return Response(status=status.HTTP_204_NO_CONTENT)

    def put(self, request, *args, **kwargs):
        item = get_object_or_404(request.event.items.all(), pk=kwargs.get("item"))

        try:
            instance = item.vacc_autosched_config
        except ItemConfig.DoesNotExist:
            instance = ItemConfig(item=item)

        s = ItemConfigSerializer(instance=instance, data=request.data)
        s.is_valid(raise_exception=True)
        with transaction.atomic():
            s.save()
            item.log_action(
                "pretix.event.item.changed", user=self.request.user, data=request.data
            )
        s = ItemConfigSerializer(instance=instance)
        return Response(s.data)
